//*****************************************************
//       File:  ClockFrame.java
//        Dir:  c:users\gordon\cis421\animation
//     Author:  HGG. Updated by Prof. Spiegel, 03/26/05
//   Computer:  PCs and Suns supporting Xwindows
//   Compiler:  Java 1.2 ; Updated to 1.5
//    Purpose:  To demonstrate Frames, Threads, 
//              Calendar, and Text Animation
//
//*****************************************************

import java.net.*;
import java.awt.*;
import java.util.*;
import java.awt.event.*;
import java.text.DateFormat;
import extra.*;
public class ClockFrame extends Frame 
                        implements Runnable, ActionListener 
{
    private Thread clockThread = null;
    private Toolkit toolkit = Toolkit.getDefaultToolkit();
    private Calendar cal, calAlarm;
    private Date theTime, dateAlarm = null;
    private DateFormat dateFormatter;
    private int clockWidth = 150;
    private int clockHeight = 105;
    private Label time;
    private Label alarm;
    private MenuBar mb = new MenuBar();
    private Menu clockMenu = new Menu("Clock");
    private MenuItem[] clockItem = 
    {
         new MenuItem("Start"),
         new MenuItem("Stop"),
         new MenuItem("Set Alarm")
    };

    public ClockFrame (String title) 
    {
        super(title);
        setMenuBar(mb);
        mb.add(clockMenu);

        // add clock menu items and register listeners
        for (int i=0; i< clockItem.length; i++) 
        {
          clockMenu.add(clockItem[i]);
          clockItem[i].addActionListener(this);
        }
        setSize(clockWidth, clockHeight);
        cal = Calendar.getInstance();		// Date & time
        theTime = cal.getTime();		// theTime holds the time

	// Object to use for formatting the time
        dateFormatter = DateFormat.getTimeInstance();
	
	// Labels for the GUI
        time = new Label(" Time: " + dateFormatter.format(theTime));
        alarm = new Label(" Alarm: Not Set");

	// Listen for window closing event
        addWindowListener(new CloseClock() );
	
	// Default is border layout
        add(time,"North");
        add(alarm, "South");
        setBackground(Color.yellow);
    }

    public void startTheClock() 
    {   // Toolkit has lots of interesting routines. Check it in the API
	// size of screen in pixels
        Dimension d = toolkit.getScreenSize();  
	// Put the clock all the way to the right (how to center it?)
        setLocation( (d.width- clockWidth), clockHeight );
        setVisible(true);
        if (clockThread == null) 
        {
            clockThread = new Thread(this, "Clock");
            clockThread.start();
        }
    }

    // Inner class to sound alarm
    class SoundAlarm extends Thread
    {
	public void run()
	{   for(int i = 0; i<10;i++) 
		toolkit.beep();
   	}
    } // class


    // Executes when Thread start() is called
    public void run() 
    {
	Thread myThread = Thread.currentThread();
        while (clockThread == myThread) 
        {   // From HGG's extras package
	    QuestionDialog alarmDialog = null;

	    // If the alarm is set...   and the time is after alarm time//
            if( (dateAlarm != null) && (theTime.compareTo(dateAlarm) >= 0) )
            { // make a sound (hmmm...kinda weak)
//              for(int i = 0; i<10;i++) toolkit.beep();
	      SoundAlarm SA=new SoundAlarm();
	      SA.start();

	      // Set up a dialog for the alarm
              String [] answers = {"Yes", "No"};
              if(alarmDialog == null)
                 alarmDialog = 
                   new QuestionDialog(this, "Turn Alarm Off?", answers);
              Dimension d = toolkit.getScreenSize();
             
              alarmDialog.setBackground(Color.red);
	      // So, that's how to center it....
              alarmDialog.setLocation( (d.width- clockWidth)/2,
                                       (d.height -clockHeight)/2 );
              alarmDialog.setVisible(true);
              if( alarmDialog.getAnswer().equals("Yes") )
                  resetAlarm();
            }
            repaint();
            try 
            {
                Thread.sleep(1000);  // once a second...
            } catch (InterruptedException e){ }
        } // while
    } // run

    public void paint(Graphics g) 
    {   // when it's time to redraw the clock...
        cal = Calendar.getInstance();
        theTime = cal.getTime();
        time.setText(" Time: " + dateFormatter.format(theTime));
    }

    public void stop() 
    {
        clockThread = null;
    }
 
    public void actionPerformed(ActionEvent e) 
    {  
      Object source = e.getSource();
	//*************************************************
	//Clock Menu Items
	//*************************************************

	// Start Clock
      if(source == clockItem[0])
      {
       startTheClock();
      }

	// stop clock
      else if(source == clockItem[1])
      { 
         stop();
         setVisible(false);
         clockItem[2].setLabel("Set Alarm");
      }

	// Set Alarm
      else if(source == clockItem[2])
      {  
         if( clockItem[2].getLabel().equals("Set Alarm") )
         {
            StringTokenizer strings;
            String ans;

	    // Another out of HGG's extras; how to do in swing?
            InputDialog input = 
                  new InputDialog(this, "Enter time as hh:mm:AM_or_PM");
            input.setVisible(true);
            ans = input.getAnswer();  // readLine()
            input.setVisible(false);

	    // tokenize string using : as separator
            strings = new StringTokenizer(ans, ":");
            if( strings.countTokens() == 3)
            { // Process alarm time; should be in try-catch
              alarm.setText(" Alarm set at " + ans);
              int hour = Integer.parseInt(strings.nextToken());
              int minutes = Integer.parseInt(strings.nextToken());
              String amPM = strings.nextToken();
              calAlarm = Calendar.getInstance();
              calAlarm.set(Calendar.HOUR, hour);
              calAlarm.set(Calendar.MINUTE, minutes);
              calAlarm.set(Calendar.SECOND, 0);
              calAlarm.set(Calendar.AM_PM, (amPM.equals("AM")?0:1) );
              dateAlarm = calAlarm.getTime();
              clockItem[2].setLabel("Reset Alarm");  // change menu item
            }
         }
         else resetAlarm();
      }
    }

    // Called when alarm isreset via yes in dialog box
    private void resetAlarm()
    {
       alarm.setText(" Alarm: Not Set");
       clockItem[2].setLabel("Set Alarm");
       dateAlarm = null;
    }



    class CloseClock extends WindowAdapter
    {
       public void windowClosing(WindowEvent we)
       {
         stop();
         setVisible(false);
       }
    }

    public static void main(String [] args)
    {
        ClockFrame cf = new ClockFrame("Clock");
        cf.startTheClock();
    }
}

